/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File resource.c : Initialisation of runtime resources
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/param.h>

#ifdef HAS_UNAME
#include <sys/utsname.h>
#endif

#ifdef VGA_DISPLAY
#include <vgakeyboard.h>
#else
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#endif
#include "z80.h"
#include "version.h"
#include "sam.h"
#include "resource.h"
#include "util.h"

#ifndef USRLIBDIR
#define USRLIBDIR "/usr/lib/X11"
#endif

#ifndef LIBDIR
#define LIBDIR ".:/usr/local/lib/SimCoupe"
#endif

#ifndef MAXPATHLEN
#define MAXPATHLEN 1024
#endif

#define MAX_CHARS 80
#define MAX_DISP_LEN 256

#define SIMCOUPE_MIN(x,y) (x<y?x:y)
#define NELEM(a) (sizeof(a)/sizeof(a[0]))

/* The resource block definition */

cfg_block SimCoupeResources;

void set_default_resources(void);

static void
usage(void)
{
	printf("SimCoupe is copyright (c) 1996 Allan Skillman\n");
	printf("SimCoupe comes with no warranty; see the file COPYRIGHT for details\n");

/* The NEC EWS C compiler claims it's ANSI, but doesn't allow string 
 * literal concatenation.  Pretty braindead and useless, if you ask me.
 */
#ifndef nec_ews
	printf("\nUsage: simcoupe [options]\n"
		"Available options:\n"
		"   -help\t\tdisplay this message\n"
		"   -version\t\tdisplay the current version\n"
		"   -rom{0,1} <fname>\tuse the specified file as SAM ROM{0,1} image\n"
		"   -fd{1,2} <fname>\tlink the specified file to SimCoupe's virtual disk drive\n"
		"   -libDir <path>\tspecify library directories\n"
		"   -flashing <bool>\tenable mode 1/2 flashing attributes [true]\n"
		"   -iFreq <n>\t\tInterrupts occur every n instructions\n"
#ifdef SLOWDOWN
		"   -slowdown <int>\t\tspecify slowdown factor\n"
		"   -keySlower <Scancode>\tslow down the emulator\n"
		"   -keyFaster <Scancode>\tspeed up the emulator\n"
#endif
#ifdef JOY
		"   -joystick <bool>\tenable/disable joystick [false]\n"
		"   -joyDev <path>\tspecify joystick device [/dev/js0]\n"
		"   -tolerance <int>\tspecify joystick displacement tolerance [20]\n"
#endif
		"   -externalMem <n>\tNumber of megabytes of external memory [0]\n"
/* X11 specific resources */
#ifndef VGA_DISPLAY
		"   -display <disp_name>\tuse the specified X display\n"
		"   -font <fontname>\tuse the specified X font [fixed]\n"
		"   -geometry <g.spec.>\tspecify window position\n"
		"   -rr <n>\t\trefresh screen every <n> interrupts (MITSHM) [1]\n"
		"   -private\t\tforce a private colormap\n"
		"   -mitshm \t\ttry to use MIT-SHM\n"
#endif
	       );

#else /* nec_ews */
	printf("Sorry, no help available until NEC fix their C compiler.\n");
#endif

	exit(0);
}


static void
show_version(void)
{
	printf("This is SimCoupe v%d.%d pl%d\n", SIMCOUPE_Major, SIMCOUPE_Minor, SIMCOUPE_Patch);
#ifndef nec_ews
	printf("Compile-time options:\n"
#ifdef VGA_DISPLAY
		   " *  Linux VGA display used\n"
#else
		   " *  X11 display used\n"
#endif
#ifdef INTEL_ASM
		   " *  Intel 80x86 assembly optimisations present.\n"
#endif
#if defined(MITSHM) && !defined(VGA_DISPLAY)
		   " *  MIT-SHM image transfer is possible.\n"
#endif
#ifdef JOY
		   " *  Linux analogue joystick emulation is possible.\n"
#endif
#ifdef PSEUDO_IO
		   " *  Character input/output is emulated via ports.\n"
#endif
#ifdef SLOWDOWN
	           " *  Speed control is enabled\n"
#endif
	);
#endif /* nec_ews */
	exit(0);
}


static int
is_true(char *str)
{
	return ((str!=NULL) ? strncasecmp(str, "true", 4) == 0
			   || strncasecmp(str, "on", 2)   == 0
			   || strncasecmp(str, "yes", 3)  == 0 : 0);
}

static int
scan_int(char *str)
{
  int i;
  
  if (!sscanf(str, "%d", &i))
  	i = -1;
  return(i);
}

void
set_default_resources(void)
{
	SetCfg(recalibrate,False);
	SetCfgStr(rom0,"sam_rom0.rom");
	SetCfgStr(rom1,"sam_rom1.rom");
	SetCfgStr(lib_dir, LIBDIR);
	SetCfgStr(disk_1,"Disk1.dsk");
	SetCfgStr(disk_2,"Disk2.dsk");
	SetCfg(ifreq, 11000);
	SetCfg(flashing, True);
#ifdef SLOWDOWN
	SetCfg(slowdown, 0);		/* slowdown factor for very fast machines */
#endif
#ifdef JOY
	SetCfg(use_js, False);		/* try to use a joystick */
	SetCfg(js_tolerance, 20);	/* joystick displacement tolerance */
	SetCfgStr(js_device, "/dev/js0");	/* joystick device name (/dev/js[0|1]) */
#endif
	SetCfg(externalMem, 0);			/* No. Mb of external memory */
	SetCfg(rr, 1);

#ifndef VGA_DISPLAY
	SetCfg(rr, 1);
	SetCfgStr(display, "");
	SetCfgStr(geometry, "");
	SetCfgStr(font, DEF_FONT);
	SetCfg(use_mitshm, 1);
	SetCfg(priv_cmap, 1);
#endif
	
}


int
get_resources(int *argcp, char **argvp)
{
	int iarg;

	/* set default resources */
	
	set_default_resources();

	/* Simple argument parser - Maybe I'll expand this in the */
	/* future */

	iarg = *argcp;
	while (--iarg) {
		if (!strcmp(*++argvp, "-help")) {
			usage();
			exit(0);
		}
		else if (!strcmp(*argvp, "-version")) {
			show_version();
			exit(0);
		}
		else if (!strcmp(*argvp, "-rom0") && (--iarg)) {
			 SetCfgStr(rom0,*++argvp);
		}
		else if (!strcmp(*argvp, "-rom1") && (--iarg)) {
			 SetCfgStr(rom1,*++argvp);
		}
		else if (!strcmp(*argvp, "-libdir") && (--iarg)) {
			 SetCfgStr(lib_dir,*++argvp);
		}
		else if (!strcmp(*argvp, "-fd1") && (--iarg)) {
			 SetCfgStr(disk_1,*++argvp);
		}
		else if (!strcmp(*argvp, "-fd2") && (--iarg)) {
			 SetCfgStr(disk_2,*++argvp);
		}
		else if (!strcmp(*argvp, "-iFreq") && (--iarg)) {
			 SetCfg(ifreq,scan_int(*++argvp));
		}
		else if (!strcmp(*argvp, "-flashing") && (--iarg)) {
			 SetCfg(flashing,is_true(*++argvp));
		}
#ifdef SLOWDOWN
		else if (!strcmp(*argvp, "-slowdown") && (--iarg)) {
			 SetCfg(slowdown,scan_int(*++argvp));
		}
#endif	
#ifdef JOY
		else if (!strcmp(*argvp, "-joystick") && (--iarg)) {
			 SetCfg(use_js, is_true(*++argvp));
		}
		else if (!strcmp(*argvp, "-tolerance") && (--iarg)) {
			 SetCfgStr(js_tolerance, scan_int(*++argvp));
		}
		else if (!strcmp(*argvp, "-joyDev") && (--iarg)) {
			 SetCfgStr(js_device, scan_int(*++argvp));
		}
#endif	
		else if (!strcmp(*argvp, "-externalMem") && (--iarg)) {
			 SetCfg(externalMem,scan_int(*++argvp));
		}	
#ifndef VGA_DISPLAY
		else if (!strcmp(*argvp, "-rr") && (--iarg)) {
			 SetCfg(rr,scan_int(*++argvp));
		}	
		else if (!strcmp(*argvp, "-display") && (--iarg)) {
			 SetCfgStr(display,*++argvp);
		}	
		else if (!strcmp(*argvp, "-geom") && (--iarg)) {
			 SetCfgStr(geometry,*++argvp);
		}	
		else if (!strcmp(*argvp, "-font") && (--iarg)) {
			 SetCfgStr(font,*++argvp);
		}	
		else if (!strcmp(*argvp, "-private")) {
			 SetCfg(priv_cmap,True);
		}	
		else if (!strcmp(*argvp, "-mitshm") && (--iarg)) {
			 SetCfg(use_mitshm,is_true(*++argvp));
		}	

#endif
		else {
			usage();
			exit(0);
		}
	}	

	return 0;
}
